@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of Radicale Server - Calendar Server
-# Copyright © 2008-2012 Guillaume Ayoub
 # Copyright © 2008 Nicolas Kandel
 # Copyright © 2008 Pascal Halter
+# Copyright © 2008-2013 Guillaume Ayoub
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,17 +39,28 @@ import wsgiref.simple_server
 # pylint: disable=F0401,E0611
 try:
     from http import client
-    from urllib.parse import quote, unquote, urlparse
+    from urllib.parse import unquote, urlparse
 except ImportError:
     import httplib as client
-    from urllib import quote, unquote
+    from urllib import unquote
     from urlparse import urlparse
 # pylint: enable=F0401,E0611
 
-from radicale import acl, config, ical, log, storage, xmlutils
+from . import auth, config, ical, log, rights, storage, xmlutils
 
 
-VERSION = "0.7.1"
+VERSION = "0.8"
+
+# Standard "not allowed" response that is returned when an authenticated user
+# tries to access information they don't have rights to
+NOT_ALLOWED = (client.FORBIDDEN, {}, None)
+
+# Standard "authenticate" response that is returned when a user tries to access
+# non-public information w/o submitting proper authentication credentials
+WRONG_CREDENTIALS = (
+    client.UNAUTHORIZED,
+    {"WWW-Authenticate": "Basic realm=\"%s\"" % config.get("server", "realm")},
+    None)
 
 
 class HTTPServer(wsgiref.simple_server.WSGIServer, object):
@@ -119,7 +130,8 @@ class Application(object):
     def __init__(self):
         """Initialize application."""
         super(Application, self).__init__()
-        self.acl = acl.load()
+        auth.load()
+        rights.load()
         storage.load()
         self.encoding = config.get("encoding", "request")
         if config.getboolean("logging", "full_environment"):
@@ -169,6 +181,64 @@ class Application(object):
         trailing_slash = "" if uri == "/" else trailing_slash
         return uri + trailing_slash
 
+    def collect_allowed_items(self, items, user):
+        """Get items from request that user is allowed to access."""
+        read_last_collection_allowed = None
+        write_last_collection_allowed = None
+        read_allowed_items = []
+        write_allowed_items = []
+
+        for item in items:
+            if isinstance(item, ical.Collection):
+                if rights.read_authorized(user, item):
+                    log.LOGGER.debug(
+                        "%s has read access to collection %s" %
+                        (user or "Anonymous", item.url or "/"))
+                    read_last_collection_allowed = True
+                    read_allowed_items.append(item)
+                else:
+                    log.LOGGER.debug(
+                        "%s has NO read access to collection %s" %
+                        (user or "Anonymous", item.url or "/"))
+                    read_last_collection_allowed = False
+
+                if rights.write_authorized(user, item):
+                    log.LOGGER.debug(
+                        "%s has write access to collection %s" %
+                        (user or "Anonymous", item.url or "/"))
+                    write_last_collection_allowed = True
+                    write_allowed_items.append(item)
+                else:
+                    log.LOGGER.debug(
+                        "%s has NO write access to collection %s" %
+                        (user or "Anonymous", item.url or "/"))
+                    write_last_collection_allowed = False
+            else:
+                # item is not a collection, it's the child of the last
+                # collection we've met in the loop. Only add this item
+                # if this last collection was allowed.
+                if read_last_collection_allowed:
+                    log.LOGGER.debug(
+                        "%s has read access to item %s" %
+                        (user or "Anonymous", item.name))
+                    read_allowed_items.append(item)
+                else:
+                    log.LOGGER.debug(
+                        "%s has NO read access to item %s" %
+                        (user or "Anonymous", item.name))
+
+                if write_last_collection_allowed:
+                    log.LOGGER.debug(
+                        "%s has write access to item %s" %
+                        (user or "Anonymous", item.name))
+                    write_allowed_items.append(item)
+                else:
+                    log.LOGGER.debug(
+                        "%s has NO write access to item %s" %
+                        (user or "Anonymous", item.name))
+
+        return read_allowed_items, write_allowed_items
+
     def __call__(self, environ, start_response):
         """Manage a request."""
         log.LOGGER.info("%s request at %s received" % (
@@ -176,9 +246,17 @@ class Application(object):
         headers = pprint.pformat(self.headers_log(environ))
         log.LOGGER.debug("Request headers:\n%s" % headers)
 
-        # Sanitize request URI
-        environ["PATH_INFO"] = self.sanitize_uri(environ["PATH_INFO"])
-        log.LOGGER.debug("Sanitized path: %s", environ["PATH_INFO"])
+        base_prefix = config.get("server", "base_prefix")
+        if environ["PATH_INFO"].startswith(base_prefix):
+            # Sanitize request URI
+            environ["PATH_INFO"] = self.sanitize_uri(
+                "/%s" % environ["PATH_INFO"][len(base_prefix):])
+            log.LOGGER.debug("Sanitized path: %s", environ["PATH_INFO"])
+        else:
+            # Request path not starting with base_prefix, not allowed
+            log.LOGGER.debug(
+                "Path not starting with prefix: %s", environ["PATH_INFO"])
+            environ["PATH_INFO"] = None
 
         # Get content
         content_length = int(environ.get("CONTENT_LENGTH") or 0)
@@ -189,77 +267,45 @@ class Application(object):
         else:
             content = None
 
+        path = environ["PATH_INFO"]
+
         # Find collection(s)
-        items = ical.Collection.from_path(
-            environ["PATH_INFO"], environ.get("HTTP_DEPTH", "0"))
+        items = ical.Collection.from_path(path, environ.get("HTTP_DEPTH", "0"))
 
         # Get function corresponding to method
         function = getattr(self, environ["REQUEST_METHOD"].lower())
 
-        # Check rights
-        if not items or not self.acl or function == self.options:
-            # No collection, or no acl, or OPTIONS request: don't check rights
-            status, headers, answer = function(environ, items, content, None)
+        # Ask authentication backend to check rights
+        authorization = environ.get("HTTP_AUTHORIZATION", None)
+
+        if authorization:
+            authorization = \
+                authorization.lstrip("Basic").strip().encode("ascii")
+            user, password = self.decode(
+                base64.b64decode(authorization), environ).split(":", 1)
         else:
-            # Ask authentication backend to check rights
-            authorization = environ.get("HTTP_AUTHORIZATION", None)
+            user = password = None
 
-            if authorization:
-                auth = authorization.lstrip("Basic").strip().encode("ascii")
-                user, password = self.decode(
-                    base64.b64decode(auth), environ).split(":")
-            else:
-                user = password = None
+        if not items or function == self.options or \
+                auth.is_authenticated(user, password):
 
-            last_collection_allowed = None
-            allowed_items = []
-            for item in items:
-                if not isinstance(item, ical.Collection):
-                    # item is not a colleciton, it's the child of the last
-                    # collection we've met in the loop. Only add this item if
-                    # this last collection was allowed.
-                    if last_collection_allowed:
-                        allowed_items.append(item)
-                    continue
+            read_allowed_items, write_allowed_items = \
+                self.collect_allowed_items(items, user)
 
-                # item is a collection
-                collection = item
-                if collection.owner in acl.PUBLIC_USERS:
-                    log.LOGGER.info("Public collection")
-                    allowed_items.append(collection)
-                    last_collection_allowed = True
-                else:
-                    log.LOGGER.info(
-                        "Checking rights for collection owned by %s" % (
-                            collection.owner or "nobody"))
-                    if self.acl.has_right(collection.owner, user, password):
-                        log.LOGGER.info(
-                            "%s allowed" % (user or "Anonymous user"))
-                        allowed_items.append(collection)
-                        last_collection_allowed = True
-                    else:
-                        log.LOGGER.info(
-                            "%s refused" % (user or "Anonymous user"))
-                        last_collection_allowed = False
-
-            if allowed_items:
-                # Collections and items found
+            if read_allowed_items or write_allowed_items or \
+                    function == self.options or not items:
+                # Collections found, or OPTIONS request, or no items at all
                 status, headers, answer = function(
-                    environ, allowed_items, content, user)
-            elif user and last_collection_allowed is None:
-                # Good user and no collections found, redirect user to home
-                location = "/%s/" % str(quote(user))
-                log.LOGGER.info("redirecting to %s" % location)
-                status = client.FOUND
-                headers = {"Location": location}
-                answer = "Redirecting to %s" % location
+                    environ, read_allowed_items, write_allowed_items, content,
+                    user)
             else:
-                # Unknown or unauthorized user
-                status = client.UNAUTHORIZED
-                headers = {
-                    "WWW-Authenticate":
-                    "Basic realm=\"Radicale Server - Password Required\""}
-                answer = None
+                # Good user but has no rights to any of the given collections
+                status, headers, answer = NOT_ALLOWED
+        else:
+            # Unknown or unauthorized user
+            log.LOGGER.info(
+                "%s refused" % (user or "Anonymous user"))
+            status, headers, answer = WRONG_CREDENTIALS
 
         # Set content length
         if answer:
@@ -267,6 +313,10 @@ class Application(object):
                 "Response content:\n%s" % self.decode(answer, environ))
             headers["Content-Length"] = str(len(answer))
 
+        if config.has_section("headers"):
+            for key in config.options("headers"):
+                headers[key] = config.get("headers", key)
+
         # Start response
         status = "%i %s" % (status, client.responses.get(status, "Unknown"))
         log.LOGGER.debug("Answer status: %s" % status)
@@ -278,9 +328,13 @@ class Application(object):
     # All these functions must have the same parameters, some are useless
     # pylint: disable=W0612,W0613,R0201
 
-    def delete(self, environ, collections, content, user):
+    def delete(self, environ, read_collections, write_collections, content,
+               user):
         """Manage DELETE request."""
-        collection = collections[0]
+        if not len(write_collections):
+            return client.PRECONDITION_FAILED, {}, None
+
+        collection = write_collections[0]
 
         if collection.path == environ["PATH_INFO"].strip("/"):
             # Path matching the collection, the collection must be deleted
@@ -301,7 +355,7 @@ class Application(object):
         # No item or ETag precondition not verified, do not delete item
         return client.PRECONDITION_FAILED, {}, None
 
-    def get(self, environ, collections, content, user):
+    def get(self, environ, read_collections, write_collections, content, user):
         """Manage GET request.
 
         In Radicale, GET requests create collections when the URL is not
@@ -312,10 +366,14 @@ class Application(object):
         # Display a "Radicale works!" message if the root URL is requested
         if environ["PATH_INFO"] == "/":
             headers = {"Content-type": "text/html"}
-            answer = "<!DOCTYPE html>\n<title>Radicale</title>Radicale works!"
+            answer = b"<!DOCTYPE html>\n<title>Radicale</title>Radicale works!"
             return client.OK, headers, answer
 
-        collection = collections[0]
+        if not len(read_collections):
+            return NOT_ALLOWED
+
+        collection = read_collections[0]
+
         item_name = xmlutils.name_from_path(environ["PATH_INFO"], collection)
 
         if item_name:
@@ -332,7 +390,15 @@ class Application(object):
         else:
             # Create the collection if it does not exist
             if not collection.exists:
-                collection.write()
+                if collection in write_collections:
+                    log.LOGGER.debug(
+                        "Creating collection %s" % collection.name)
+                    collection.write()
+                else:
+                    log.LOGGER.debug(
+                        "Collection %s not available and could not be created "
+                        "due to missing write rights" % collection.name)
+                    return NOT_ALLOWED
 
             # Get whole collection
             answer_text = collection.text
@@ -345,14 +411,21 @@ class Application(object):
         answer = answer_text.encode(self.encoding)
         return client.OK, headers, answer
 
-    def head(self, environ, collections, content, user):
+    def head(self, environ, read_collections, write_collections, content,
+             user):
         """Manage HEAD request."""
-        status, headers, answer = self.get(environ, collections, content, user)
+        status, headers, answer = self.get(
+            environ, read_collections, write_collections, content, user)
         return status, headers, None
 
-    def mkcalendar(self, environ, collections, content, user):
+    def mkcalendar(self, environ, read_collections, write_collections, content,
+                   user):
         """Manage MKCALENDAR request."""
-        collection = collections[0]
+        if not len(write_collections):
+            return NOT_ALLOWED
+
+        collection = write_collections[0]
+
         props = xmlutils.props_from_request(content)
         timezone = props.get("C:calendar-timezone")
         if timezone:
@@ -361,12 +434,17 @@ class Application(object):
         with collection.props as collection_props:
             for key, value in props.items():
                 collection_props[key] = value
-        collection.write()
+            collection.write()
         return client.CREATED, {}, None
 
-    def mkcol(self, environ, collections, content, user):
+    def mkcol(self, environ, read_collections, write_collections, content,
+              user):
         """Manage MKCOL request."""
-        collection = collections[0]
+        if not len(write_collections):
+            return NOT_ALLOWED
+
+        collection = write_collections[0]
+
         props = xmlutils.props_from_request(content)
         with collection.props as collection_props:
             for key, value in props.items():
@@ -374,9 +452,14 @@ class Application(object):
         collection.write()
         return client.CREATED, {}, None
 
-    def move(self, environ, collections, content, user):
+    def move(self, environ, read_collections, write_collections, content,
+             user):
         """Manage MOVE request."""
-        from_collection = collections[0]
+        if not len(write_collections):
+            return NOT_ALLOWED
+
+        from_collection = write_collections[0]
+
         from_name = xmlutils.name_from_path(
             environ["PATH_INFO"], from_collection)
         if from_name:
@@ -389,9 +472,12 @@ class Application(object):
                     to_path, to_name = to_url.rstrip("/").rsplit("/", 1)
                     to_collection = ical.Collection.from_path(
                         to_path, depth="0")[0]
-                    to_collection.append(to_name, item.text)
-                    from_collection.remove(from_name)
-                    return client.CREATED, {}, None
+                    if to_collection in write_collections:
+                        to_collection.append(to_name, item.text)
+                        from_collection.remove(from_name)
+                        return client.CREATED, {}, None
+                    else:
+                        return NOT_ALLOWED
                 else:
                     # Remote destination server, not supported
                     return client.BAD_GATEWAY, {}, None
@@ -402,35 +488,49 @@ class Application(object):
             # Moving collections, not supported
             return client.FORBIDDEN, {}, None
 
-    def options(self, environ, collections, content, user):
+    def options(self, environ, read_collections, write_collections, content,
+                user):
         """Manage OPTIONS request."""
         headers = {
-            "Allow": "DELETE, HEAD, GET, MKCALENDAR, MKCOL, MOVE, " \
-                "OPTIONS, PROPFIND, PROPPATCH, PUT, REPORT",
+            "Allow": ("DELETE, HEAD, GET, MKCALENDAR, MKCOL, MOVE, "
+                      "OPTIONS, PROPFIND, PROPPATCH, PUT, REPORT"),
             "DAV": "1, 2, 3, calendar-access, addressbook, extended-mkcol"}
         return client.OK, headers, None
 
-    def propfind(self, environ, collections, content, user):
+    def propfind(self, environ, read_collections, write_collections, content,
+                 user):
         """Manage PROPFIND request."""
+        # Rights is handled by collection in xmlutils.propfind
         headers = {
             "DAV": "1, 2, 3, calendar-access, addressbook, extended-mkcol",
             "Content-Type": "text/xml"}
+        collections = set(read_collections + write_collections)
         answer = xmlutils.propfind(
             environ["PATH_INFO"], content, collections, user)
         return client.MULTI_STATUS, headers, answer
 
-    def proppatch(self, environ, collections, content, user):
+    def proppatch(self, environ, read_collections, write_collections, content,
+                  user):
         """Manage PROPPATCH request."""
-        collection = collections[0]
-        answer = xmlutils.proppatch(environ["PATH_INFO"], content, collection)
+        if not len(write_collections):
+            return NOT_ALLOWED
+
+        collection = write_collections[0]
+
+        answer = xmlutils.proppatch(
+            environ["PATH_INFO"], content, collection)
         headers = {
             "DAV": "1, 2, 3, calendar-access, addressbook, extended-mkcol",
             "Content-Type": "text/xml"}
         return client.MULTI_STATUS, headers, answer
 
-    def put(self, environ, collections, content, user):
+    def put(self, environ, read_collections, write_collections, content, user):
         """Manage PUT request."""
-        collection = collections[0]
+        if not len(write_collections):
+            return NOT_ALLOWED
+
+        collection = write_collections[0]
+
         collection.set_mimetype(environ.get("CONTENT_TYPE"))
         headers = {}
         item_name = xmlutils.name_from_path(environ["PATH_INFO"], collection)
@@ -438,18 +538,19 @@ class Application(object):
 
         # Evolution bug workaround
         etag = environ.get("HTTP_IF_MATCH", "").replace("\\", "")
+        match = environ.get("HTTP_IF_NONE_MATCH", "") == "*"
         if (not item and not etag) or (
-                item and ((etag or item.etag) == item.etag)):
+                item and ((etag or item.etag) == item.etag) and not match):
             # PUT allowed in 3 cases
             # Case 1: No item and no ETag precondition: Add new item
             # Case 2: Item and ETag precondition verified: Modify item
             # Case 3: Item and no Etag precondition: Force modifying item
             xmlutils.put(environ["PATH_INFO"], content, collection)
-            status = client.NO_CONTENT if item else client.CREATED
-            # Try to return the etag in the header
-            # If the added item does't have the same name as the one given by
-            # the client, then there's no obvious way to generate an etag, we
-            # can safely ignore it.
+            status = client.CREATED
+            # Try to return the etag in the header.
+            # If the added item does't have the same name as the one given
+            # by the client, then there's no obvious way to generate an
+            # etag, we can safely ignore it.
             new_item = collection.get_item(item_name)
             if new_item:
                 headers["ETag"] = new_item.etag
@@ -458,10 +559,16 @@ class Application(object):
             status = client.PRECONDITION_FAILED
         return status, headers, None
 
-    def report(self, environ, collections, content, user):
+    def report(self, environ, read_collections, write_collections, content,
+               user):
         """Manage REPORT request."""
-        collection = collections[0]
+        if not len(read_collections):
+            return NOT_ALLOWED
+
+        collection = read_collections[0]
+
         headers = {"Content-Type": "text/xml"}
+
         answer = xmlutils.report(environ["PATH_INFO"], content, collection)
         return client.MULTI_STATUS, headers, answer
 
