@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of Radicale Server - Calendar Server
-# Copyright © 2008-2012 Guillaume Ayoub
+# Copyright © 2008-2013 Guillaume Ayoub
 # Copyright © 2008 Nicolas Kandel
 # Copyright © 2008 Pascal Halter
 #
@@ -45,18 +45,21 @@ INITIAL_CONFIG = {
         "ssl": "False",
         "certificate": "/etc/radicale/server.crt",
         "key": "/etc/radicale/private/server.key",
-        "dns_lookup": "True"},
+        "dns_lookup": "True",
+        "base_prefix": "/",
+        "realm": "Radicale - Password Required"},
     "encoding": {
         "request": "utf-8",
         "stock": "utf-8"},
-    "acl": {
+    "auth": {
         "type": "None",
         "public_users": "public",
         "private_users": "private",
         "htpasswd_filename": "/etc/radicale/users",
         "htpasswd_encryption": "crypt",
-        "imap_auth_host_name": "localhost",
-        "imap_auth_host_port": "143",
+        "imap_hostname": "localhost",
+        "imap_port": "143",
+        "imap_ssl": "False",
         "ldap_url": "ldap://localhost:389/",
         "ldap_base": "ou=users,dc=example,dc=com",
         "ldap_attribute": "uid",
@@ -65,11 +68,17 @@ INITIAL_CONFIG = {
         "ldap_password": "",
         "ldap_scope": "OneLevel",
         "pam_group_membership": "",
-        "courier_socket": ""},
+        "courier_socket": "",
+        "http_url": "",
+        "http_user_parameter": "",
+        "http_password_parameter": ""},
+    "rights": {
+        "type": "None",
+        "file": ""},
     "storage": {
         "type": "filesystem",
-        "filesystem_folder": "/var/db//radicale/calendars",
-        "git_folder": "/var/db/radicale/calendars"},
+        "filesystem_folder": "/var/db/radicale/calendars",
+        "database_url": ""},
     "logging": {
         "config": "/etc/radicale/logging",
         "debug": "False",
